#!/usr/bin/env node

/**
 * ZAI MCP Server - AI-to-AI Loop System with Multi-Provider Support
 * Supports OpenRouter, Anthropic, and DeepSeek APIs with automatic failover
 */

import { Server } from '@modelcontextprotocol/sdk/server/index.js';
import { StdioServerTransport } from '@modelcontextprotocol/sdk/server/stdio.js';
import { CallToolRequestSchema, ListToolsRequestSchema } from '@modelcontextprotocol/sdk/types.js';
import { MultiProviderAI } from './multiProviderAI.js';
import { ZAIDataCollector } from './data-collection-system.js';
import { AIVotingManager } from './aiVotingManager.js';

class ZAIMCPServer {
    constructor() {
        this.server = new Server(
            {
                name: 'zai-mcp-server',
                version: '1.0.0',
            },
            {
                capabilities: {
                    tools: {},
                },
            }
        );

        // Initialize AI providers, voting system, and data collection
        try {
            this.multiAI = new MultiProviderAI();
            this.votingManager = new AIVotingManager(this.multiAI);
            this.dataCollector = new ZAIDataCollector();
        } catch (error) {
            console.error('❌ Failed to initialize components:', error);
            throw error;
        }
        
        // Active loops storage
        this.activeLoops = new Map();
        this.loopCounter = 0;
        
        // Agent state management
        this.agentBusy = false;
        this.pendingResponses = [];

        this.setupToolHandlers();
        this.setupErrorHandling();

        console.error('🤖 ZAI MCP Server initialized with multi-provider AI support');
    }

    setupToolHandlers() {
        // List available tools
        this.server.setRequestHandler(ListToolsRequestSchema, async () => ({
            tools: [
                {
                    name: 'activate_infinite_loop',
                    description: 'Start AI-to-AI improvement loops for continuous development',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Activation message with topic (format: "actloop [topic]")'
                            },
                            aiToAi: {
                                type: 'boolean',
                                description: 'Enable AI-to-AI communication mode',
                                default: true
                            },
                            interval: {
                                type: 'number',
                                description: 'Loop interval in milliseconds',
                                default: 5000
                            },
                            maxIterations: {
                                type: 'number',
                                description: 'Maximum iterations before auto-stop',
                                default: 999999
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'stop_ai_loops',
                    description: 'Stop AI-to-AI loops',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string',
                                description: 'Stop command (e.g., "stploop", "stoploop")'
                            }
                        },
                        required: ['message']
                    }
                },
                {
                    name: 'list_active_loops',
                    description: 'List all currently active infinite loops',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_provider_status',
                    description: 'Get status of all AI providers',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'reset_ai_providers',
                    description: 'Reset all failed AI providers and API keys',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                },
                {
                    name: 'get_ai_prompts',
                    description: 'Get AI-generated prompts ready for VSCode assistant',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Maximum number of prompts to return',
                                default: 5
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'acknowledge_agent_response',
                    description: 'Acknowledge agent response to continue loop',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            loopId: {
                                type: 'string',
                                description: 'Loop ID to acknowledge'
                            },
                            agentResponse: {
                                type: 'string',
                                description: 'Agent response text (optional)'
                            }
                        },
                        required: ['loopId']
                    }
                },
                {
                    name: 'ai_voting_request',
                    description: 'Submit a prompt for multi-model AI voting and consensus',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            prompt: {
                                type: 'string',
                                description: 'The prompt to be evaluated by multiple AI models'
                            },
                            panel: {
                                type: 'string',
                                description: 'Voting panel to use (general, coding, reasoning, premium)',
                                default: 'general'
                            },
                            strategy: {
                                type: 'string',
                                description: 'Voting strategy (majority, consensus, weighted, unanimous)',
                                default: 'consensus'
                            },
                            maxAgents: {
                                type: 'number',
                                description: 'Maximum number of AI agents to participate',
                                default: 5
                            },
                            enableDebug: {
                                type: 'boolean',
                                description: 'Enable detailed voting debug information',
                                default: false
                            }
                        },
                        required: ['prompt']
                    }
                },
                {
                    name: 'get_voting_history',
                    description: 'Get recent AI voting session history',
                    inputSchema: {
                        type: 'object',
                        properties: {
                            limit: {
                                type: 'number',
                                description: 'Number of recent sessions to return',
                                default: 10
                            }
                        },
                        required: []
                    }
                },
                {
                    name: 'get_agent_performance',
                    description: 'Get performance statistics for AI voting agents',
                    inputSchema: {
                        type: 'object',
                        properties: {},
                        required: []
                    }
                }
            ]
        }));

        // Handle tool calls
        this.server.setRequestHandler(CallToolRequestSchema, async (request) => {
            const { name, arguments: args } = request.params;

            try {
                switch (name) {
                    case 'activate_infinite_loop':
                        return await this.activateInfiniteLoop(args);
                    
                    case 'stop_ai_loops':
                        return await this.stopAILoops(args);
                    
                    case 'list_active_loops':
                        return await this.listActiveLoops();
                    
                    case 'get_ai_provider_status':
                        return await this.getAIProviderStatus();
                    
                    case 'reset_ai_providers':
                        return await this.resetAIProviders();
                    
                    case 'get_ai_prompts':
                        return await this.getAIPrompts(args);
                    
                    case 'acknowledge_agent_response':
                        return await this.acknowledgeAgentResponse(args);

                    case 'ai_voting_request':
                        return await this.handleVotingRequest(args);

                    case 'get_voting_history':
                        return await this.getVotingHistory(args);

                    case 'get_agent_performance':
                        return await this.getAgentPerformance();

                    default:
                        throw new Error(`Unknown tool: ${name}`);
                }
            } catch (error) {
                console.error(`❌ Tool error [${name}]:`, error.message);
                return {
                    content: [
                        {
                            type: 'text',
                            text: `Error executing ${name}: ${error.message}`
                        }
                    ]
                };
            }
        });
    }

    async activateInfiniteLoop(args) {
        const { message, aiToAi = true, interval = 5000, maxIterations = 999999 } = args;
        
        if (!message || !message.toLowerCase().includes('actloop')) {
            throw new Error('Invalid activation message. Use format: "actloop [topic]"');
        }

        const topic = message.replace(/actloop\s*/i, '').trim();
        const loopId = `loop-${++this.loopCounter}-${Date.now()}`;
        
        const loop = {
            id: loopId,
            topic,
            aiToAi,
            interval,
            maxIterations,
            currentIteration: 0,
            startTime: new Date().toISOString(),
            status: 'active',
            lastResponse: null
        };

        this.activeLoops.set(loopId, loop);
        
        // Start the loop
        this.startLoop(loop);
        
        console.log(`🔄 AI-to-AI loop activated: ${loopId} - Topic: ${topic}`);
        
        return {
            content: [
                {
                    type: 'text',
                    text: `✅ AI-to-AI loop activated successfully!\n\nLoop ID: ${loopId}\nTopic: ${topic}\nMode: ${aiToAi ? 'AI-to-AI' : 'Regular'}\nInterval: ${interval}ms\nMax Iterations: ${maxIterations}\n\n🤖 The AI agents will now continuously improve and generate prompts for your development workflow.`
                }
            ],
            result: { loopId, topic, status: 'active' }
        };
    }

    async startLoop(loop) {
        if (loop.status !== 'active') return;
        
        try {
            loop.currentIteration++;
            
            if (loop.currentIteration > loop.maxIterations) {
                loop.status = 'completed';
                this.activeLoops.delete(loop.id);
                console.log(`🏁 Loop ${loop.id} completed after ${loop.maxIterations} iterations`);
                return;
            }

            // Generate AI prompt for the topic
            const prompt = `Iteration ${loop.currentIteration}: Improve and enhance the development workflow for: ${loop.topic}.

            Provide specific, actionable suggestions that a developer can implement. Focus on:
            1. Code quality improvements
            2. Development efficiency
            3. Best practices
            4. Tool recommendations
            5. Workflow optimizations

            Keep the response practical and implementable.`;

            console.log(`🤖 Loop ${loop.id} - Iteration ${loop.currentIteration}: Starting AI voting...`);

            // Use voting system for better responses
            const votingResult = await this.votingManager.conductVoting(prompt, {
                panel: 'general',
                strategy: 'consensus',
                maxAgents: 3,
                enableDebug: false
            });

            const response = {
                content: votingResult.selectedResponse,
                model: votingResult.consensus.winner.name,
                provider: 'voting-consensus',
                responseTime: votingResult.votingSession.duration,
                votingResults: votingResult.consensus
            };

            loop.lastResponse = response;
            
            // Collect data for training
            await this.dataCollector.collectInteraction({
                type: 'ai-to-ai-loop',
                prompt,
                response: response.content,
                context: { topic: loop.topic, iteration: loop.currentIteration },
                success: true,
                responseTime: response.responseTime || 2000,
                loopIteration: loop.currentIteration,
                model: response.model,
                sessionId: loop.id
            });

            console.log(`✅ Loop ${loop.id} - Iteration ${loop.currentIteration} completed`);
            
            // Schedule next iteration
            setTimeout(() => this.startLoop(loop), loop.interval);
            
        } catch (error) {
            console.error(`❌ Loop ${loop.id} error:`, error.message);
            
            // Collect error data
            await this.dataCollector.collectInteraction({
                type: 'ai-to-ai-loop',
                prompt: `Loop ${loop.id} iteration ${loop.currentIteration}`,
                response: null,
                context: { topic: loop.topic, iteration: loop.currentIteration },
                success: false,
                errors: [error.message],
                loopIteration: loop.currentIteration,
                sessionId: loop.id
            });
            
            // Retry after longer interval
            setTimeout(() => this.startLoop(loop), loop.interval * 2);
        }
    }

    async stopAILoops(args) {
        const { message } = args;
        
        if (!message || !['stploop', 'stoploop', 'stop loop'].some(cmd => 
            message.toLowerCase().includes(cmd))) {
            throw new Error('Invalid stop command. Use "stploop", "stoploop", or "stop loop"');
        }

        const stoppedLoops = [];
        for (const [loopId, loop] of this.activeLoops) {
            loop.status = 'stopped';
            stoppedLoops.push(loopId);
        }
        
        this.activeLoops.clear();
        
        console.log(`🛑 Stopped ${stoppedLoops.length} AI loops`);
        
        return {
            content: [
                {
                    type: 'text',
                    text: `🛑 Successfully stopped ${stoppedLoops.length} AI-to-AI loops.\n\nStopped loops: ${stoppedLoops.join(', ')}`
                }
            ],
            result: { stoppedLoops, count: stoppedLoops.length }
        };
    }

    async listActiveLoops() {
        const loops = Array.from(this.activeLoops.values()).map(loop => ({
            id: loop.id,
            topic: loop.topic,
            status: loop.status,
            currentIteration: loop.currentIteration,
            maxIterations: loop.maxIterations,
            startTime: loop.startTime,
            interval: loop.interval
        }));

        return {
            content: [
                {
                    type: 'text',
                    text: `📊 Active AI-to-AI Loops: ${loops.length}\n\n${loops.map(loop => 
                        `🔄 ${loop.id}\n  Topic: ${loop.topic}\n  Status: ${loop.status}\n  Iteration: ${loop.currentIteration}/${loop.maxIterations}\n  Started: ${loop.startTime}\n`
                    ).join('\n')}`
                }
            ],
            result: { loops, count: loops.length }
        };
    }

    async getAIProviderStatus() {
        const status = this.multiAI.getStatus();
        
        return {
            content: [
                {
                    type: 'text',
                    text: `🤖 AI Provider Status:\n\n${JSON.stringify(status, null, 2)}`
                }
            ],
            result: status
        };
    }

    async resetAIProviders() {
        this.multiAI.resetFailedProviders();
        
        return {
            content: [
                {
                    type: 'text',
                    text: '🔄 AI providers and API keys have been reset successfully!'
                }
            ],
            result: { status: 'reset_complete' }
        };
    }

    async getAIPrompts(args) {
        const { limit = 5 } = args;
        
        // Generate prompts based on active loops
        const prompts = [];
        let promptCount = 0;
        
        for (const loop of this.activeLoops.values()) {
            if (promptCount >= limit) break;
            
            if (loop.lastResponse) {
                prompts.push({
                    id: `prompt-${loop.id}-${loop.currentIteration}`,
                    content: loop.lastResponse.content,
                    topic: loop.topic,
                    iteration: loop.currentIteration,
                    timestamp: new Date().toISOString(),
                    model: loop.lastResponse.model,
                    provider: loop.lastResponse.provider
                });
                promptCount++;
            }
        }
        
        return {
            content: [
                {
                    type: 'text',
                    text: `📝 Generated ${prompts.length} AI prompts ready for VSCode assistant`
                }
            ],
            result: { prompts, count: prompts.length }
        };
    }

    async acknowledgeAgentResponse(args) {
        const { loopId, agentResponse } = args;
        
        const loop = this.activeLoops.get(loopId);
        if (!loop) {
            throw new Error(`Loop ${loopId} not found`);
        }
        
        console.log(`✅ Agent response acknowledged for loop ${loopId}`);
        
        return {
            content: [
                {
                    type: 'text',
                    text: `✅ Agent response acknowledged for loop ${loopId}`
                }
            ],
            result: { loopId, status: 'acknowledged' }
        };
    }

    async handleVotingRequest(args) {
        const { prompt, panel = 'general', strategy = 'consensus', maxAgents = 5, enableDebug = false } = args;

        console.log(`🗳️ Processing voting request: Panel=${panel}, Strategy=${strategy}`);

        try {
            const votingResult = await this.votingManager.conductVoting(prompt, {
                panel,
                strategy,
                maxAgents,
                enableDebug
            });

            // Collect voting data for training
            await this.dataCollector.collectInteraction({
                type: 'ai-voting',
                prompt,
                response: votingResult.selectedResponse,
                context: {
                    panel,
                    strategy,
                    consensus: votingResult.consensus.confidence,
                    totalAgents: votingResult.agentResponses.length
                },
                success: true,
                responseTime: votingResult.votingSession.duration,
                model: votingResult.consensus.winner.name,
                sessionId: votingResult.votingSession.id
            });

            return {
                content: [
                    {
                        type: 'text',
                        text: `🗳️ AI Voting Complete!\n\n**Selected Response:**\n${votingResult.selectedResponse}\n\n**Consensus Details:**\n- Winner: ${votingResult.consensus.winner.name}\n- Confidence: ${votingResult.consensus.confidence}%\n- Strategy: ${strategy}\n- Panel: ${panel}\n- Participating Agents: ${votingResult.agentResponses.length}\n- Total Votes: ${votingResult.consensus.totalVotes}\n\n${votingResult.consensus.hasConsensus ? '✅ Strong consensus reached!' : '⚠️ Weak consensus - consider additional review.'}`
                    }
                ],
                result: {
                    selectedResponse: votingResult.selectedResponse,
                    consensus: votingResult.consensus,
                    votingSession: votingResult.votingSession,
                    agentCount: votingResult.agentResponses.length
                }
            };

        } catch (error) {
            console.error('❌ Voting request failed:', error.message);

            return {
                content: [
                    {
                        type: 'text',
                        text: `❌ AI voting failed: ${error.message}\n\nFalling back to single model response...`
                    }
                ]
            };
        }
    }

    async getVotingHistory(args) {
        const { limit = 10 } = args;

        const history = this.votingManager.getVotingHistory(limit);

        const historyText = history.map(session =>
            `🗳️ ${session.id} (${session.timestamp})\n` +
            `  Panel: ${session.panel} | Strategy: ${session.strategy}\n` +
            `  Agents: ${session.selectedAgents.length} | Confidence: ${session.consensus?.confidence || 'N/A'}%\n` +
            `  Duration: ${session.duration}ms\n`
        ).join('\n');

        return {
            content: [
                {
                    type: 'text',
                    text: `📊 Recent AI Voting History (${history.length} sessions):\n\n${historyText || 'No voting history available.'}`
                }
            ],
            result: { history, count: history.length }
        };
    }

    async getAgentPerformance() {
        const performance = this.votingManager.getAgentPerformance();

        const performanceText = Object.entries(performance).map(([agentId, stats]) =>
            `🤖 ${agentId}:\n` +
            `  Success Rate: ${stats.successRate}%\n` +
            `  Total Votes: ${stats.totalVotes}\n` +
            `  Correct Votes: ${stats.correctVotes}\n`
        ).join('\n');

        return {
            content: [
                {
                    type: 'text',
                    text: `📈 AI Agent Performance Statistics:\n\n${performanceText || 'No performance data available.'}`
                }
            ],
            result: performance
        };
    }

    setupErrorHandling() {
        this.server.onerror = (error) => {
            console.error('❌ MCP Server error:', error);
        };

        process.on('SIGINT', async () => {
            console.log('\n🛑 Shutting down ZAI MCP Server...');
            
            // Stop all active loops
            for (const loop of this.activeLoops.values()) {
                loop.status = 'stopped';
            }
            this.activeLoops.clear();
            
            await this.server.close();
            process.exit(0);
        });
    }

    async start() {
        try {
            const transport = new StdioServerTransport();
            await this.server.connect(transport);
            console.error('🚀 ZAI MCP Server running on stdio transport');
        } catch (error) {
            console.error('❌ Failed to start MCP server:', error);
            throw error;
        }
    }
}

// Start the server
if (import.meta.url === `file://${process.argv[1]}`) {
    const server = new ZAIMCPServer();
    server.start().catch((error) => {
        console.error('❌ Failed to start ZAI MCP Server:', error);
        process.exit(1);
    });
}

export { ZAIMCPServer };
